import Note from '../../models/Note'
import Folder from '../../models/Folder'

export interface Plugin {
	type: string
}

export interface Paginated<T> extends Array<T> {
	totalLength: number,
}

export type PaginationOptions = {
	skip?: number,
	take?: number,
}

export interface StoragePlugin extends Plugin {
	type: 'storage'
	saveNote(newNote: Note): Promise<Note>
	loadNote(noteId: string): Promise<Note>
	loadAllNotes(options?: PaginationOptions): Promise<Paginated<Note>>
	saveFolder(newFolder: Folder): Promise<Folder>
	loadFolder(folderId: string): Promise<Folder>
}

export type StoragePluginOptions = {

}

export abstract class StoragePluginBase {
	readonly type = 'storage' as const
}

export default class PluginService {
	private readonly registered: Plugin[] = []

	register(plugin: Plugin) {
		this.registered.push(plugin)
	}

	getAll() {
		return this.registered.slice()
	}
}
