import {GetServerSideProps, NextPage} from 'next'
import NoteView from '../../../components/templates/NoteView'
import {QueryFragment, Subpage} from '../../../utils/query'
import Note from '../../../models/Note'
import Folder from '../../../models/Folder'

type Props = {
	subpage: Subpage,
	notes: Note[],
	currentFolder?: Folder,
	subfolders: Folder[],
	currentNote: Note,
}

const Page: NextPage<Props> = ({
	subpage,
	notes,
	currentFolder,
	subfolders,
	currentNote,
}) => {
	return (
		<NoteView
			subpage={subpage}
			notes={notes}
			subfolders={subfolders}
			currentFolder={currentFolder}
			currentNote={currentNote}
		/>
	);
};

export default Page

export const getServerSideProps: GetServerSideProps = async (ctx) => {
	const { [QueryFragment.SUBPAGE]: subpage = '', noteId, } = ctx.query
	const currentFolder: Folder = {
		id: '0',
		name: 'Root Folder',
		description: 'Default location of your notes.',
	}
	const authorUser = {
		id: '0',
		profile: {
			id: '0',
			email: 'hello@example.com',
			links: [],
		},
		username: 'johndoe'
	}
	const notes: Note[] = [
		{
			id: '0',
			title: 'Hello',
			authorUser: authorUser,
			contentVersions: [
				{
					id: '0',
					content: {
						version: '0.3.2',
						markups: [
							['b'],
							['i'],
						],
						atoms: [],
						cards: [],
						sections: [
							[1, "p", [
								[0, [], 0, "Example with no markup"],
								[0, [0], 1, "Example wrapped in b tag (opened markup #0), 1 closed markup"],
								[0, [1], 0, "Example opening i tag (opened markup with #1, 0 closed markups)"],
								[0, [], 1, "Example closing i tag (no opened markups, 1 closed markup)"],
								[0, [1, 0], 1, "Example opening i tag and b tag, closing b tag (opened markups #1 and #0, 1 closed markup [closes markup #0])"],
								[0, [], 1, "Example closing i tag, (no opened markups, 1 closed markup [closes markup #1])"],
							]]
						],
					},
					createdAt: new Date().toISOString(),
				},
			],
			folder: currentFolder,
		}
	]
	const currentNote: Note = notes.find(n => n.id === noteId)
	const subfolders: Folder[] = [
		{
			id: '1',
			name: 'Child Folder',
			description: 'Where we put other notes',
		}
	]
	return {
		props: {
			subpage,
			notes,
			subfolders,
			currentFolder,
			currentNote,
		},
	}
}
