import {GetServerSideProps, NextPage} from 'next'
import NoteView from '../../../components/templates/NoteView'
import {QueryFragment, Subpage} from '../../../utils/query'
import Note from '../../../models/Note'
import Folder from '../../../models/Folder'

type Props = {
	subpage: Subpage,
	notes: Note[],
	currentFolder?: Folder,
	subfolders: Folder[],
}

const Page: NextPage<Props> = ({
	subpage,
	notes,
	currentFolder,
	subfolders,
}) => {
	return (
		<NoteView
			subpage={subpage}
			notes={notes}
			subfolders={subfolders}
			currentFolder={currentFolder}
		/>
	);
};

export default Page

export const getServerSideProps: GetServerSideProps = async (ctx) => {
	const { [QueryFragment.SUBPAGE]: subpage = '' } = ctx.query
	const currentFolder: Folder = {
		id: '0',
		name: 'Root Folder',
		description: 'Default location of your notes.',
	}
	const authorUser = {
		id: '0',
		profile: {
			id: '0',
			email: 'hello@example.com',
			links: [],
		},
		username: 'johndoe'
	}
	const notes: Note[] = [
		{
			id: '0',
			title: 'Hello',
			authorUser: authorUser,
			contentVersions: [
				{
					id: '0',
					content: {
						version: '0.3.2',
						markups: [],
						atoms: [],
						cards: [],
						sections: [],
					},
					createdAt: new Date().toISOString(),
				},
			],
			folder: currentFolder,
		}
	]
	const subfolders: Folder[] = [
		{
			id: '1',
			name: 'Child Folder',
			description: 'Where we put other notes',
		}
	]
	return {
		props: {
			subpage,
			notes,
			subfolders,
			currentFolder,
		},
	}
}
