enum MarkerType {
	TEXT = 0,
	ATOM = 1,
}

enum SectionType {
	MARKUP = 1,
	IMAGE = 2,
	LIST = 3,
	CARD = 10,
}

// TODO!!!!!
//   Try parsing BBCode before converting it to Mobiledoc and vv
const generateBBCodeAST = (bbcode: string) => (
	bbcode
		.trim()
		.split('')
		.reduce(
			(parserState, c) => {
				const {tokenStack, tagStack, tree} = parserState
				const [lastToken] = tokenStack.slice(-1)
				switch (c) {
				case '[':
					return {
						...parserState,
						tokenStack: [
							...tokenStack,
							'',
						],
					}
				case ']':
					if (
						lastToken.startsWith('/')
						&& lastToken.slice('/'.length) === tagStack.slice(-1)[0]
					) {
						return {
							...parserState,
							tokenStack: tokenStack.slice(0, -1),
							tagStack: tagStack.slice(0, -1),
						}
					}
					return {
						...parserState,
						tokenStack: tokenStack.slice(0, -1),
						tagStack: [
							...tagStack,
							lastToken,
						]
					}
				default:
					break
				}

				return {
					...parserState,
					tokenStack: [
						...tokenStack.slice(0, -1),
						lastToken + c,
					],
				}
			},
			{
				tree: [],
				tokenStack: [''],
				tagStack: [],
			}
		)
)

const convertBBCodeToMobiledoc = (bbcode: string, { components, version = '0.3.2,' }) => {
	return bbcode
		.trim()
		.split('')
		.reduce(
			(parserState, c) => {
				switch (c) {
				case '[':
					return {
						...parserState,
						tokenStack: [
							...parserState.tokenStack,
							'',
						]
					}
				case ']':
					if (parserState.lastToken.startsWith('/')) {
						return {
							...parserState,
							// pop token stack
							tokenStack: parserState.tokenStack.slice(0, -1),
							tagStack: parserState.tagStack.slice(0, -1),
						}
					}
					return {
						...parserState,
						// pop token stack
						tokenStack: parserState.tokenStack.slice(0, -1),
						tagStack: [
							...parserState.tagStack,
							parserState.lastToken,
						],
						lastToken: '',
					}
				}

				let [lastSection = [SectionType.MARKUP, 'p', []]] = parserState.state.sections.slice(-1)
				let [sectionType = SectionType.MARKUP, sectionTag = 'p', sectionMarkers = []] = lastSection
				let [lastMarker = [MarkerType.TEXT, [], 0, '']] = sectionMarkers.slice(-1)
				let [textTypeIdentifier, openMarkupsIndexes, numberOfClosedMarkups, value] = lastMarker

				return {
					...parserState,
					sections: [
						...parserState.state.sections,
						[
							sectionType,
							sectionTag,
							[
								...sectionMarkers,
								[
									textTypeIdentifier,
									openMarkupsIndexes,
									numberOfClosedMarkups,
									value + parserState.lastCharacter,
								],
							],
						],
					],
					lastCharacter: c,
				}
			},
			{
				state: {
					version,
					markups: [],
					atoms: [],
					cards: [],
					sections: [],
				},
				lastCharacter: '',
				lastToken: '',
				tokenStack: [],
				tagStack: [],
			}
		)
		.state
}

const convertMobiledocToBBCode = (mobiledoc: any, { components }) => {

}

export const deserialize = (text: string) => {
	return convertBBCodeToMobiledoc(text, { components: [] })
}

export const serialize = (data: any) => {
	return convertMobiledocToBBCode(data, { components: [] })
}
